-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local taxiFrameWidth = 0;

local flightNode     = {};
local flightList     = {};
local collapsedZones = {};

local nodeMapping =
{
  [L["Shattered Sun Staging Area"]] =
  {
    zone = L["Isle of Quel'Danas"],
    loc  = L["Shattered Sun Staging Area"]
  },
  [L["Dalaran"]] =
  {
    zone = L["Crystalsong Forest"],
    loc  = L["Dalaran"]
  },
  [L["Transitus Shield, Coldarra"]] =
  {
    zone = L["Borean Tundra"],
    loc  = L["Transitus Shield, Coldarra"]
  },
  [L["Hellfire Peninsula, The Dark Portal"]] =
  {
    zone = L["Hellfire Peninsula"],
    loc  = L["The Dark Portal"]
  },
  [L["Hellfire Peninsula, The Dark Portal, Alliance"]] =
  {
    zone = L["Hellfire Peninsula"],
    loc  = L["The Dark Portal"]
  },
  [L["Hellfire Peninsula, The Dark Portal, Horde"]] =
  {
    zone = L["Hellfire Peninsula"],
    loc  = L["The Dark Portal"]
  },
  [L["Acherus: The Ebon Hold"]] =
  {
    zone = L["Eastern Plaguelands"],
    loc  = L["Acherus: The Ebon Hold"]
  },
  [L["The Exodar"]] =
  {
    zone = L["Azuremyst Isle"],
    loc  = L["The Exodar"]
  }
};

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------


-- Function to update the state of the button that toggles the displaying of
-- the 'Flights' panel
--
local function buttonToggleUpdateState()
  -- Local Variables
  local button = _G["NigglesFlightsToggleButton"];

  -- Set the texture of the button
  if (NigglesFlights:IsShown()) then
    button:SetNormalTexture(
      "Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up");
    button:SetPushedTexture(
      "Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down");
    button:SetDisabledTexture(
      "Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Disabled");
  else
    button:SetNormalTexture(
      "Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up");
    button:SetPushedTexture(
      "Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down");
    button:SetDisabledTexture(
      "Interface\\Buttons\\UI-SpellbookIcon-NextPage-Disabled");
  end

  return;
end

--
-- Function to create the list used in the scrolled frame. The list includes
-- elements for the zones and handles the hiding of locations if the zone is
-- collapsed
--
local function flightListCreate()
  -- Local Variables
  local currentZone = "";
  local isCollapsed;
  local numNodes = #(flightNode);
  local searchText = _G["NigglesFlightsSearchBox"]:GetText();
  local addZone;
  local addLocations;
  local idx;

  -- Check if there is any search text to match against
  if ((searchText ~= SEARCH) and
      (searchText ~= "") and
      (searchText ~= nil)) then
      searchText = string.lower(searchText);
  else
    searchText = nil;
  end

  -- Clear the list of flight nodes
  wipe(flightList);

  -- Process all the flight nodes
  nodeIdx = 1;
  while (nodeIdx <= numNodes) do
    -- Initialise some variables
    currentZone  = flightNode[nodeIdx].zone;
    isCollapsed  = (collapsedZones[currentZone] == true);
    addZone      = false;
    addLocations = false;

    -- Check if the zone should be added
    if (searchText ~= nil) then
      if (string.find(string.lower(currentZone), searchText,
           1, true) ~= nil) then
        addZone      = true;
        addLocations = true;
      else
        idx = nodeIdx;
        while ((idx <= numNodes) and 
               (flightNode[idx].zone == currentZone)) do
          if (string.find(string.lower(flightNode[idx].location), searchText,
               1, true) ~= nil) then
            addZone = true;
            break;
          end
          idx = idx+1;
        end
      end
    else
      addZone      = true;
      addLocations = true;
    end

    -- Add the zone, if required
    if (addZone) then
      table.insert(flightList, -nodeIdx);
    end

    -- Add the locations, if required
    while ((nodeIdx <= numNodes) and
           (flightNode[nodeIdx].zone == currentZone)) do
      if ((not isCollapsed) and
          ((addLocations) or
           (string.find(string.lower(flightNode[nodeIdx].location),
             searchText, 1, true) ~= nil))) then
        table.insert(flightList, nodeIdx);
      end
      nodeIdx = nodeIdx+1;
    end
  end

  return;
end

--
-- Function to compare 2 flight nodes
--
local function flightNodeSort(first, second)
  -- Local Variables
  local retValue;

  -- Work out which order the nodes should be
  if (first.zone ~= second.zone) then
    retValue = (naturalString(first.zone) < naturalString(second.zone));
  else
    retValue =
      (naturalString(first.location) < naturalString(second.location));
  end

  return retValue;
end

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to update the Flights List in the Niggles Flights panel.
--
function NigglesFlightsUpdate()
  -- Local Variables
  local scrollFrame = NigglesFlightsScrollFrame;
  local button;
  local listIdx;
  local numRows = #(flightList);
  local numButtons = #scrollFrame.buttons;
  local topOffset = HybridScrollFrame_GetOffset(scrollFrame);
  local nodeIdx;
  local totalHeight;
  local displayedHeight;
  
  -- Make sure the top offset is valid
  if (topOffset > numRows) then
    topOffset = numRows;
    HybridScrollFrame_SetOffset(scrollFrame, topOffset);
  end

  -- Update all the buttons
  for idx = 1, numButtons do
    -- Get the current button
    button = scrollFrame.buttons[idx];

    -- Work out the index of the flight node to display in the button
    listIdx = topOffset+idx;
    if (listIdx <= numRows) then
      -- Set the button's ID
      button:SetID(listIdx);

      -- Check if the node is for a zone
      nodeIdx = flightList[listIdx];
      if (nodeIdx < 0) then
        button.text:SetText(flightNode[-nodeIdx].zone);
        button:SetNormalFontObject(GameFontNormalLeft);
        button:Enable();
        if (collapsedZones[flightNode[-nodeIdx].zone] == true) then
          button:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
        else
          button:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
        end
        button.hilite:SetTexture("Interface\\Buttons\\UI-PlusButton-Hilight");
      else
        button.text:SetText(flightNode[nodeIdx].location);
        if (NigglesIsCity(flightNode[nodeIdx].location)) then
          button:SetNormalFontObject(GameFontNormalLeftGreen);
        else
          button:SetNormalFontObject(GameFontHighlight);
        end
        if (flightNode[nodeIdx].type == "REACHABLE") then
          button:Enable();
        else
          button:Disable();
        end
        button:SetNormalTexture("");
        button.hilite:SetTexture("");
      end

      -- Show the button
      button:Show();
    else
      -- Update the button
      button:Disable();
      button:Hide();
    end
  end

  -- Update the scroll frame's height
  totalHeight     = numRows*scrollFrame.buttons[1]:GetHeight();
  displayedHeight = numButtons*scrollFrame.buttons[1]:GetHeight();
  HybridScrollFrame_Update(scrollFrame, totalHeight, displayedHeight);

  return;
end

--
-- Function to update the Niggles Flights panel when it is shown
--
function NigglesFlightsOnShow()
  -- Local Variables
  local nodeType;
  local nodeName;
  local numNodes = NumTaxiNodes();
  local zoneName;
  local zoneLoc;

  -- Clear the flight nodes
  wipe(flightNode);

  -- Get the info for all the nodes
  for nodeIdx = 1, numNodes do
    nodeType = TaxiNodeGetType(nodeIdx);
    if (nodeType ~= "NONE") then
      -- Work out the zone and location to use for the node
      nodeName = TaxiNodeName(nodeIdx);
      if (nodeMapping[nodeName] ~= nil) then
        zoneName = nodeMapping[nodeName].zone;
        zoneLoc  = nodeMapping[nodeName].loc;
      else
        zoneLoc, zoneName = string.match(nodeName, "(.*[^,]+), ([^,]*)");
        if ((zoneLoc == nil) or (zoneName == nil)) then
          zoneName = nodeName;
          zoneLoc  = nodeName;
        end
      end

      -- Add the node to the table
      table.insert(flightNode,
        {
          index    = nodeIdx,
          zone     = L[zoneName],
          location = L[zoneLoc],
          type     = nodeType
        });
    end
  end

  -- Sort the table
  table.sort(flightNode, flightNodeSort);

  -- Create the flight list
  flightListCreate();

  -- Set the scroll frames to the top
  HybridScrollFrame_SetOffset(NigglesFlightsScrollFrame, 0);
  NigglesFlightsUpdate();

  return;
end

--
-- Function used to update the Flights list, based on the search text
--
function NigglesFlightsSearchUpdate()
  -- Create a new flight list, based on the search text
  flightListCreate();

  -- Set the scroll frames to the top
  HybridScrollFrame_SetOffset(NigglesFlightsScrollFrame, 0);
  NigglesFlightsUpdate();

  return;
end
  
--
-- Function to process a button click in the Flights list
--
function NigglesFlightsButtonOnClick(button)
  -- Local variables
  local listIdx = button:GetID();

  -- Take a flight to the flight node clicked
  if (listIdx > 0) then
    if (flightList[listIdx] < 0) then
      if (collapsedZones[flightNode[-flightList[listIdx]].zone] == nil) then
        collapsedZones[flightNode[-flightList[listIdx]].zone] = true;
      else
        collapsedZones[flightNode[-flightList[listIdx]].zone] = nil;
      end
      flightListCreate();
      NigglesFlightsUpdate();
    else
      TakeTaxiNode(flightNode[flightList[listIdx]].index);
      PlaySound("igMainMenuOptionCheckBoxOn");
    end
  end

  return;
end

--
-- Function to handle OnEnter events for a button in the Flights List
--
function NigglesFlightsButtonOnEnter(button)
  -- Local variables
  local listIdx = button:GetID();

  -- Make the Taxi Frame show the route
  if ((listIdx > 0) and (flightList[listIdx] >=  0)) then
    TaxiNodeOnButtonEnter(
      _G["TaxiButton"..flightNode[flightList[listIdx]].index]);
  end

  return;
end

--
-- Function called when the Niggles Flights panel is hidden
--
function NigglesFlightsOnHide(frame)
  -- Update the state of the toggle button
  buttonToggleUpdateState();

  return;
end

--
-- Function called when the Niggles Flights panel is loaded
--
function NigglesFlightsOnLoad(frame)
  -- Get the width of the Flights panel
  taxiFrameWidth = TaxiFrame:GetWidth();

  -- Initialise the scroll frame
  frame.scrollFrame.update = NigglesFlightsUpdate;
  HybridScrollFrame_CreateButtons(frame.scrollFrame,
    "NigglesFlightsButtonTemplate", 1, 0);
  frame.scrollFrame.scrollBar.doNotHide = true;

  return;
end

--
-- Function to toggle the displaying of the extension to the Taxi panel
--
function NigglesFlightsToggleOnClick(button)
  -- Toggle the displaying of the Flights panel
  if (NigglesFlights:IsShown()) then
    NigglesFlights:Hide();
    TaxiFrame:SetWidth(taxiFrameWidth);
    TaxiFrame.InsetBg:SetPoint("BOTTOMRIGHT", TaxiFrame.InsetBg:GetParent(),
      -6, 4);
  else
    NigglesFlights:Show();
    TaxiFrame:SetWidth(taxiFrameWidth+NigglesFlights:GetWidth());
    TaxiFrame.InsetBg:SetPoint("BOTTOMRIGHT", TaxiFrame.InsetBg:GetParent(),
      -(6+NigglesFlights:GetWidth()), 4);
  end
  SetUIPanelAttribute(TaxiFrame, "width", TaxiFrame:GetWidth());
  UpdateUIPanelPositions(TaxiFrame);
  PlaySound("igMainMenuOptionCheckBoxOn");

  -- Update the state of the button
  buttonToggleUpdateState();

  return;
end
